@@ -4,9 +4,13 @@ import unittest as _unittest
 import collections as _collections
 import pprint as _pprint
 import math as _math
+import functools as _functools
+import operator as _operator
+import multiprocessing as _multiprocessing
+import itertools as _itertools
 
 
-__version__ = '0.0.11'
+__version__ = '0.0.12'
 
 
 class Error(Exception):
@@ -16,23 +20,47 @@ class Error(Exception):
 TICK_INTERVAL_PADDING_RATIO = 0.1
 
 
+def parallel_for(f, *indicess):
+    return reshape(_multiprocessing.Pool().starmap(f, _itertools.product(*indicess)),
+                   [len(indices) for indices in indicess])
+
+
+def reshape(xs, ns):
+    assert ns
+    assert len(xs) == _functools.reduce(_operator.mul, ns, 1)
+    return _reshape(xs, ns)
+
+
+def _reshape(xs, ns):
+    if len(ns) == 1:
+        return xs
+    return _reshape(partition(xs, ns[-1]), ns[0:-1])
+
+
+def partition(xs, n):
+    ret = []
+    for i in range(n, len(xs) + 1, n):
+        ret.append(xs[i - n:i])
+    return ret
+
+
 def memoize(f):
     cache = {}
-    def retf(*args):
+    def memoized_f(*args):
         if args in cache:
             return cache[args]
         else:
             cache[args] = retv = f(*args)
             return retv
-    retf.cache = cache
-    return retf
+    memoized_f.cache = cache
+    return memoized_f
 
 
 def profiled_memoize(f):
     cache = {}
     profile = {'new': 0,
                'hit': 0}
-    def retf(*args):
+    def profiled_memoized_f(*args):
         if args in cache:
             profile['hit'] += 1
             return cache[args]
@@ -40,9 +68,9 @@ def profiled_memoize(f):
             profile['new'] += 1
             cache[args] = retv = f(*args)
             return retv
-    retf.cache = cache
-    retf.profile = profile
-    return retf
+    profiled_memoized_f.cache = cache
+    profiled_memoized_f.profile = profile
+    return profiled_memoized_f
 
 
 def _get_interval(lx):
@@ -120,8 +148,6 @@ def list_2d(n_row, n_column, init=None):
             in range(n_row)]
 
 
-# TODO: listを返すバージョンも作る
-# TODO: 負の数でスキップを可能にする
 def make_fixed_format_parser(fields):
     """
     fields: (('density', 3, int),
@@ -165,7 +191,46 @@ class TestAction(_argparse.Action):
         parser.exit()
 
 
+def _fn_for_test_parallel_for(x, y):
+    return x, y
+
+
 class _Tester(_unittest.TestCase):
+    def test_parallel_for(self):
+        self.assertEqual(parallel_for(_fn_for_test_parallel_for, [1, 2], [3, 4, 5]), [[(1, 3), (1, 4), (1, 5)], [(2, 3), (2, 4), (2, 5)]])
+
+    def test_reshape(self):
+        with self.assertRaises(AssertionError):
+            reshape((1,), ())
+        with self.assertRaises(AssertionError):
+            reshape((1, 2, 3), (2, 2))
+
+        for xs, ns, expected in (
+                ([1, 2, 3], (3,), [1, 2, 3]),
+                ([1, 2, 3], (3, 1), [[1], [2], [3]]),
+                ([1, 2, 3, 4], (2, 2), [[1, 2], [3, 4]]),
+                ([1, 2, 3, 4, 5, 6], (2, 3), [[1, 2, 3], [4, 5, 6]]),
+                ([1, 2, 3, 4, 5, 6, 7, 8], (2, 2, 2), [[[1, 2], [3, 4]], [[5, 6], [7, 8]]]),
+        ):
+            self.assertEqual(reshape(xs, ns), expected)
+
+    def test_partition(self):
+        with self.assertRaises(ValueError):
+            partition((1, 2), 0)
+
+        for xs, n, expected in (
+                ((), -1, []),
+                ((1, 2), -1, []),
+
+                ((), 1, []),
+                ((), 2, []),
+
+                ([1, 3], 1, [[1], [3]]),
+                ([1, 4], 2, [[1, 4]]),
+                ([1, 5], 3, []),
+        ):
+            self.assertEqual(partition(xs, n), expected)
+
     def test__get_interval(self):
         with self.assertRaises(AssertionError):
             _get_interval(-1)
